"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestClient = exports.requestWithRetry = exports.userPath = exports.apiPath = void 0;
const got_1 = __importDefault(require("got"));
const util_1 = require("./util");
const apiBaseUrl = 'https://api.ps-smartfeed.cloud.petsafe.net/api/v2/', usersApiBasePath = 'https://users-api.ps-smartfeed.cloud.petsafe.net/users/', defaultRequestOptions = {
    http2: true,
    responseType: 'json',
    method: 'GET',
};
function apiPath(path) {
    return apiBaseUrl + path;
}
exports.apiPath = apiPath;
function userPath(path = '') {
    return usersApiBasePath + path;
}
exports.userPath = userPath;
function requestWithRetry(options) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = (yield got_1.default(Object.assign(Object.assign({}, defaultRequestOptions), options)));
            return response.body;
        }
        catch (e) {
            if (!e.response) {
                util_1.logError(`Failed to reach PetSafe server at ${options.url}.  ${e.message}.  Trying again in 5 seconds...`);
                yield util_1.delay(5000);
                return requestWithRetry(options);
            }
            throw e;
        }
    });
}
exports.requestWithRetry = requestWithRetry;
class RestClient {
    constructor(authOptions) {
        this.authOptions = authOptions;
    }
    request(options) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const headers = Object.assign(Object.assign({}, options.headers), { token: this.authOptions.token }), response = yield requestWithRetry(Object.assign(Object.assign({}, options), { headers }));
                return response;
            }
            catch (e) {
                const response = e.response || {}, { url } = options;
                if (response.status === 400 &&
                    ((_c = (_b = (_a = response === null || response === void 0 ? void 0 : response.data) === null || _a === void 0 ? void 0 : _a.errors) === null || _b === void 0 ? void 0 : _b.app) === null || _c === void 0 ? void 0 : _c[0]) === 'invalid-token') {
                    util_1.logError('Your token has expired!');
                }
                if (response.status === 404 && url.startsWith(apiBaseUrl)) {
                    util_1.logError('404 from endpoint ' + url);
                    throw new Error('Not found with response: ' + JSON.stringify(response.data));
                }
                util_1.logError(`Request to ${url} failed (${response.status}). ${JSON.stringify(response.data)}`);
                throw e;
            }
        });
    }
}
exports.RestClient = RestClient;
